@@ -184,8 +184,13 @@ void __eztrace_alarm_sighandler(int signo __attribute__((unused))){
 
 #ifdef __linux__
 long __ezt_alarm_interval = 0;
+int alarm_enabled = 0;
+int alarm_set = 0;
+
 void eztrace_set_alarm() {
-  if(__ezt_alarm_interval>=0) {
+  if(__ezt_alarm_interval>=0 && alarm_enabled && (! alarm_set)) {
+    alarm_set = 1;
+
     struct sigevent sevp;
     sevp.sigev_notify=SIGEV_THREAD_ID | SIGEV_SIGNAL;
     sevp.sigev_signo=SIGALRM;
@@ -215,6 +220,9 @@ void eztrace_set_alarm() {
     }
   }
 }
+#else
+void eztrace_set_alarm() {
+}
 #endif
 
 static void __eztrace_set_sighandler() {
@@ -232,6 +240,7 @@ static void __eztrace_set_sighandler() {
   res = getenv("EZTRACE_SIGALARM");
   if(res && (strncmp(res, "0", 2) != 0)) {
     /* convert from ms to ns */
+    alarm_enabled = 1;
     __ezt_alarm_interval = atoi(res)*1000000;
     printf("[EZTrace] Setting an alarm every %d ms\n", atoi(res));
     signal(SIGALRM, __eztrace_alarm_sighandler);
