@@ -224,6 +224,7 @@ enum coll_time_ids {
 struct mpi_coll_msg_t {
   enum coll_type_t type;
   uint64_t** times;
+  int data_size;
   int comm_size;
   int nb_started;
   struct ezt_mpi_comm **comm;
@@ -370,8 +371,8 @@ static struct mpi_thread_info_t* __register_thread_hook(
   for(i=0; i<MPI_ID_SIZE; i++) {
     p_info->nb_mpi_calls[i] = 0;
     p_info->total_time_mpi_calls[i] = 0;
-    p_info->enter_date_mpi_calls[i] = 0;
-    p_info->leave_date_mpi_calls[i] = 0;
+    p_info->enter_date_mpi_calls[i] = -1;
+    p_info->leave_date_mpi_calls[i] = -1;
   }
 
   /* add the hook in the thread info structure */
@@ -435,4 +436,7 @@ extern int *rank_to_trace_id;
 int is_comm_mine(int global_rank,
 		 struct ezt_mpi_comm*comm);
 
+#define IS_TIME_SET(t) ((t)!=UINT64_MAX)
+#define TIME_INIT UINT64_MAX
+
 #endif	/*  EZTRACE_CONVERT_MPI_H */
