@@ -14,6 +14,9 @@
 #include "eztrace_convert_mpi_coll.h"
 
 void init_mpi_coll_messages() {
+  hierarchical_array_attach(EZTRACE_MPI_STATS_COLL_MSG_ID,
+			    sizeof(struct coll_msg_event));
+
 }
 
 void print_coll_stats();
@@ -48,7 +51,8 @@ void __print_coll_messages(FILE*stream) {
 
 /* create a new collective message */
 static struct mpi_coll_msg_t* __new_coll_message(enum coll_type_t type,
-						 struct ezt_mpi_comm *comm)
+						 struct ezt_mpi_comm *comm,
+						 int data_size)
 {
   struct mpi_coll_msg_t *msg = malloc(sizeof(struct mpi_coll_msg_t));
   int comm_size = comm->comm_size;
@@ -57,6 +61,7 @@ static struct mpi_coll_msg_t* __new_coll_message(enum coll_type_t type,
   msg->comm_size = comm_size;
   msg->comm = malloc(sizeof(struct ezt_mpi_comm*) * comm_size);
   int i;
+  msg->data_size = data_size;
   msg->times = malloc(sizeof(uint64_t*) * comm_size);
   msg->thread_ids = malloc(sizeof(char*) * comm_size);
   msg->link_value = malloc(sizeof(char**) * comm_size);
@@ -77,7 +82,7 @@ static struct mpi_coll_msg_t* __new_coll_message(enum coll_type_t type,
 
     msg->times[i] = malloc(sizeof(uint64_t) * COLL_NB_TIMES);
     for (j = 0; j < COLL_NB_TIMES; j++)
-      msg->times[i][j] = 0;
+      msg->times[i][j] = TIME_INIT;
   }
 
   return msg;
@@ -109,7 +114,7 @@ static int __mpi_coll_msg_match(struct mpi_coll_msg_t* msg,
   if ((msg->type == type)
       && (msg->comm_size == comm->comm_size)
       && ((!msg->requests[my_rank]) || (msg->requests[my_rank] == req))
-      && (time_id < 0 || !msg->times[my_rank][time_id])) {
+      && (time_id < 0 || !IS_TIME_SET(msg->times[my_rank][time_id]))) {
     /* It looks like this message matches.
      * We need to check wether the communicator for this message is comm or
      * if it matches comm
@@ -242,6 +247,7 @@ __enter_coll(uint64_t time,
 	     enum coll_type_t type,
 	     struct ezt_mpi_comm *comm,
 	     int __attribute__((unused)) my_rank,
+	     int data_size,
 	     struct mpi_request* req,
 	     char* thread_id)
 {
@@ -251,7 +257,7 @@ __enter_coll(uint64_t time,
     /* We are the first process to arrive in this collective communication.
      * create a new collective message structure
      */
-    msg = __new_coll_message(type, comm);
+    msg = __new_coll_message(type, comm, data_size);
   }
 
   /* add this message in the list of the process pending communications */
@@ -260,6 +266,7 @@ __enter_coll(uint64_t time,
   assert(msg->comm[comm->my_rank] == NULL);
 
   /* todo: record_event(time, type, msg) */
+
   msg->nb_started++;
   msg->comm[comm->my_rank] = comm;
   assert(msg->comm[comm->my_rank] == comm);
@@ -275,6 +282,17 @@ __enter_coll(uint64_t time,
   return msg;
 }
 
+static void
+__store_finished_collective(p_eztrace_container p_cont,
+			    uint64_t time,
+			    int my_rank,
+			    struct mpi_coll_msg_t* p_msg) {
+  struct coll_msg_event* msg = hierarchical_array_new_item(p_cont, EZTRACE_MPI_STATS_COLL_MSG_ID);
+  msg->time = time;
+  msg->my_rank = my_rank;
+  msg->msg = p_msg;
+}
+
 struct mpi_coll_msg_t*
 __leave_coll(uint64_t time,
 	     enum coll_type_t type,
@@ -306,16 +324,21 @@ __leave_coll(uint64_t time,
   assert(msg->comm[comm->my_rank] == comm);
 
   if(msg->nb_started != comm->comm_size) {
-    msg->times[comm->my_rank][stop_coll] = 0;
+    msg->times[comm->my_rank][stop_coll] = TIME_INIT;
     /* some process did not reach the start of the collective. We need to wait
      * for them
      */
     return NULL;
   }
 
+  /* remove the collective from the list of pending collectives */
   ezt_list_remove(&pending_comm->token);
   free(pending_comm);
 
-  /* remove the collective from the list of pending collectives */
+  __store_finished_collective(GET_PROCESS_CONTAINER(comm->ranks[comm->my_rank]),
+			      time,
+			      comm->my_rank,
+			      msg);
+
   return msg;
 }
