@@ -17,6 +17,7 @@
 #include "eztrace_convert.h"
 #include "eztrace_convert_mpi.h"
 #include "eztrace_convert_mpi_p2p.h"
+#include "eztrace_convert_mpi_coll.h"
 #include "eztrace_hierarchical_array.h"
 #include "eztrace_array.h"
 
@@ -40,6 +41,18 @@ struct mpi_p2p_stat_t {
   struct stat_double_counter_t rwait_duration;
 };
 
+struct mpi_coll_stat_t {
+  int nb_messages;
+  /* size of sent messages */
+  struct stat_uint64_t_counter_t size;
+
+  /* time spent computing while sending/receiving messages */
+  /* TODO: add the overlapped time */
+  //  struct stat_double_counter_t overlap_duration;
+  /* time spent blocked in the collective */
+  struct stat_double_counter_t wait_duration;
+};
+
 struct __mpi_stats_matrix_item {
   uint64_t total_len;
   int nb_messages;
@@ -53,6 +66,8 @@ struct __mpi_stats_freq_item {
 struct eztrace_array_t __mpi_stats_freq;
 struct __mpi_stats_matrix_item **__mpi_stats_comm_matrix = NULL;
 
+struct __mpi_stats_matrix_item **__mpi_stats_comm_matrix_coll = NULL;
+
 /* find a len in the frequency table */
 static struct __mpi_stats_freq_item*
 __find_freq_item(int len) {
@@ -82,6 +97,12 @@ static void __update_p2p_message_stats(int src, int dest, int len,
   freq_item->nb_occur++;
 }
 
+/* update the communication matrix for collective messages */
+static void __update_coll_message_stats(int src, int dest, int len, enum coll_type_t type) {
+  __mpi_stats_comm_matrix_coll[src][dest].nb_messages++;
+  __mpi_stats_comm_matrix_coll[src][dest].total_len += len;
+}
+
 static void __print_stat_double_counter(struct stat_double_counter_t *counter,
                                         int count) {
   printf("\tmin: %lf", counter->min);
@@ -111,6 +132,14 @@ static void __init_mpi_p2p_stat_t(struct mpi_p2p_stat_t* counter) {
   __init_stat_counter(double, &counter->rwait_duration);
 }
 
+static void __init_mpi_coll_stat_t(struct mpi_coll_stat_t* counter) {
+  counter->nb_messages = 0;
+  __init_stat_counter(uint64_t, &counter->size);
+
+  //  __init_stat_counter(double, &counter->overlap_duration);
+  __init_stat_counter(double, &counter->wait_duration);
+}
+
 static void __p2p_stats_reduce_recurse(unsigned depth, int rank,
                                        p_eztrace_container p_cont) {
   assert(p_cont);
@@ -294,30 +323,6 @@ static void __p2p_stats_print_recurse(unsigned depth,
 
 }
 
-/* compute point to point messages statistics */
-static void __p2p_stats_reduce() {
-  int rank;
-  eztrace_array_create(&__mpi_stats_freq, sizeof(struct __mpi_stats_freq_item),
-                       32);
-
-  __mpi_stats_comm_matrix = malloc(
-      sizeof(struct __mpi_stats_matrix_item*) * NB_TRACES);
-  for (rank = 0; rank < NB_TRACES; rank++) {
-    __mpi_stats_comm_matrix[rank] = malloc(
-        sizeof(struct __mpi_stats_matrix_item) * NB_TRACES);
-    int i;
-    for (i = 0; i < NB_TRACES; i++) {
-      __mpi_stats_comm_matrix[rank][i].total_len = 0;
-      __mpi_stats_comm_matrix[rank][i].nb_messages = 0;
-    }
-  }
-
-  for (rank = 0; rank < NB_TRACES; rank++) {
-    p_eztrace_container p_cont = GET_PROCESS_CONTAINER(rank);
-    __p2p_stats_reduce_recurse(0, rank, p_cont);
-  }
-}
-
 /* print point to point messages statistics */
 static void __p2p_stats_print() {
   int rank;
@@ -330,7 +335,7 @@ static void __p2p_stats_print() {
 int dump_comm_matrix_nb_msg = 0;
 int dump_comm_matrix_msg_size = 0;
 
-static void __p2p_create_comm_matrix_gnuplot(const char* data_prefix) {
+static void __create_comm_matrix_gnuplot(const char* data_prefix) {
 
     char* path;
     asprintf(&path, "%s.gp", data_prefix);
@@ -353,7 +358,7 @@ static void __p2p_print_communication_matrix_nb_messages() {
 
   if (dump_comm_matrix_nb_msg) {
     char* prefix;
-    asprintf(&prefix, "%s/communication_matrix.nb_messages",
+    asprintf(&prefix, "%s/communication_matrix_p2p.nb_messages",
              eztrace_stats_get_output_dir());
     char* path;
     asprintf(&path, "%s.dat", prefix);
@@ -369,7 +374,7 @@ static void __p2p_print_communication_matrix_nb_messages() {
     fclose(out);
     printf("\tThe communication matrix (number of message) has been dumped in %s\n",
         path);
-    __p2p_create_comm_matrix_gnuplot(prefix);
+    __create_comm_matrix_gnuplot(prefix);
     free(prefix);
    free(path);
   }
@@ -380,7 +385,7 @@ static void __p2p_print_communication_matrix_message_size() {
   if (dump_comm_matrix_msg_size) {
 
     char* prefix;
-    asprintf(&prefix, "%s/communication_matrix.message_size",
+    asprintf(&prefix, "%s/communication_matrix_p2p.message_size",
              eztrace_stats_get_output_dir());
     char* path;
     asprintf(&path, "%s.dat", prefix);
@@ -396,15 +401,244 @@ static void __p2p_print_communication_matrix_message_size() {
     fclose(out);
     printf("\tThe communication matrix (message size) has been dumped in %s\n",
            path);
-    __p2p_create_comm_matrix_gnuplot(prefix);
+    __create_comm_matrix_gnuplot(prefix);
     free(path);
     free(prefix);
   }
 }
 
+
+static void __all_print_communication_matrix_nb_messages() {
+  int src, dest;
+
+  if (dump_comm_matrix_nb_msg) {
+    char* prefix;
+    asprintf(&prefix, "%s/communication_matrix_all.nb_messages",
+             eztrace_stats_get_output_dir());
+    char* path;
+    asprintf(&path, "%s.dat", prefix);
+
+    FILE *out = fopen(path, "w+");
+
+    for (src = 0; src < NB_TRACES; src++) {
+      for (dest = 0; dest < NB_TRACES; dest++) {
+        fprintf(out, "%d ", __mpi_stats_comm_matrix[src][dest].nb_messages + __mpi_stats_comm_matrix_coll[src][dest].nb_messages);
+      }
+      fprintf(out, "\n");
+    }
+    fclose(out);
+    printf("\tThe communication matrix (number of message) has been dumped in %s\n",
+        path);
+    __create_comm_matrix_gnuplot(prefix);
+    free(prefix);
+   free(path);
+  }
+}
+
+static void __all_print_communication_matrix_message_size() {
+  int src, dest;
+  if (dump_comm_matrix_msg_size) {
+
+    char* prefix;
+    asprintf(&prefix, "%s/communication_matrix_all.message_size",
+             eztrace_stats_get_output_dir());
+    char* path;
+    asprintf(&path, "%s.dat", prefix);
+
+    FILE *out = fopen(path, "w+");
+
+    for (src = 0; src < NB_TRACES; src++) {
+      for (dest = 0; dest < NB_TRACES; dest++) {
+        fprintf(out, "%ld ", __mpi_stats_comm_matrix[src][dest].total_len + __mpi_stats_comm_matrix_coll[src][dest].total_len);
+      }
+      fprintf(out, "\n");
+    }
+    fclose(out);
+    printf("\tThe communication matrix (message size) has been dumped in %s\n",
+           path);
+    __create_comm_matrix_gnuplot(prefix);
+    free(path);
+    free(prefix);
+  }
+}
+
+
+
+static void __coll_print_communication_matrix_nb_messages() {
+  int src, dest;
+
+  if (dump_comm_matrix_nb_msg) {
+    char* prefix;
+    asprintf(&prefix, "%s/communication_matrix_collective.nb_messages",
+             eztrace_stats_get_output_dir());
+    char* path;
+    asprintf(&path, "%s.dat", prefix);
+
+    FILE *out = fopen(path, "w+");
+
+    for (src = 0; src < NB_TRACES; src++) {
+      for (dest = 0; dest < NB_TRACES; dest++) {
+        fprintf(out, "%d ", __mpi_stats_comm_matrix_coll[src][dest].nb_messages);
+      }
+      fprintf(out, "\n");
+    }
+    fclose(out);
+    printf("\tThe communication matrix (number of collective messages) has been dumped in %s\n",
+        path);
+    __create_comm_matrix_gnuplot(prefix);
+    free(prefix);
+   free(path);
+  }
+}
+
+static void __coll_print_communication_matrix_message_size() {
+  int src, dest;
+  if (dump_comm_matrix_msg_size) {
+
+    char* prefix;
+    asprintf(&prefix, "%s/communication_matrix_collective.message_size",
+             eztrace_stats_get_output_dir());
+    char* path;
+    asprintf(&path, "%s.dat", prefix);
+
+    FILE *out = fopen(path, "w+");
+
+    for (src = 0; src < NB_TRACES; src++) {
+      for (dest = 0; dest < NB_TRACES; dest++) {
+        fprintf(out, "%ld ", __mpi_stats_comm_matrix_coll[src][dest].total_len);
+      }
+      fprintf(out, "\n");
+    }
+    fclose(out);
+    printf("\tThe communication matrix (collective messages size) has been dumped in %s\n",
+           path);
+    __create_comm_matrix_gnuplot(prefix);
+    free(path);
+    free(prefix);
+  }
+}
+
+
+static void __coll_stats_reduce_recurse(unsigned depth, int rank,
+                                       p_eztrace_container p_cont) {
+  assert(p_cont);
+
+  struct mpi_coll_stat_t* counter = hierarchical_array_new_item(p_cont, EZTRACE_MPI_STATS_COLL_ID);
+  assert(counter);
+  __init_mpi_coll_stat_t(counter);
+
+  unsigned i;
+  /* compute the children stats */
+  for (i = 0; i < p_cont->nb_children; i++) {
+    /* call recursively the function */
+    __coll_stats_reduce_recurse(depth + 1, rank, p_cont->children[i]);
+    struct hierarchical_array* child_array = hierarchical_array_find(
+        EZTRACE_MPI_STATS_COLL_ID, p_cont->children[i]);
+    assert(child_array);
+    assert(child_array->nb_items);
+
+    /* add the children stats to the container stats */
+    struct mpi_coll_stat_t*child_counter = ITH_ITEM(0, child_array);
+    counter->nb_messages += child_counter->nb_messages;
+
+    __reduce_stat_counter(uint64_t, &counter->size, &child_counter->size);
+
+    //    __reduce_stat_counter(double, &counter->overlap_duration,
+    //                          &child_counter->overlap_duration);
+    __reduce_stat_counter(double, &counter->wait_duration,
+                          &child_counter->wait_duration);
+  }
+
+  /* compute statistics for the current container */
+  unsigned index;
+
+  struct hierarchical_array* array = hierarchical_array_find(EZTRACE_MPI_STATS_COLL_MSG_ID, p_cont);
+  assert(array);
+  for (index = 0; index < array->nb_items; index++) {
+    /* for each completed collective... */
+    struct coll_msg_event *msg = NULL;
+    msg = ITH_ITEM(index, array);
+    assert(msg);
+
+    counter->nb_messages++;
+    __update_stat_counter(uint64_t, &counter->size, (uint64_t )msg->msg->data_size);
+    for(i=0; i<msg->msg->comm_size; i++) {
+      __update_coll_message_stats(msg->my_rank,
+				  ezt_get_global_rank(msg->msg->comm[msg->my_rank], i),
+				  msg->msg->data_size,
+				  msg->msg->type);
+    }
+
+    /* compute communication durations */
+    __update_stat_counter(
+        double,
+        &counter->wait_duration,
+        NS_TO_MS(msg->msg->times[msg->my_rank][stop_coll] - msg->msg->times[msg->my_rank][start_coll]));
+  }
+}
+
+static void __coll_stats_print_recurse(unsigned depth,
+				       p_eztrace_container p_cont) {
+  assert(p_cont);
+
+  struct hierarchical_array* array = hierarchical_array_find(
+      EZTRACE_MPI_STATS_COLL_ID, p_cont);
+  assert(array);
+
+  struct mpi_coll_stat_t* counter = ITH_ITEM(0, array);
+  assert(counter);
+
+  unsigned i;
+  if (counter->nb_messages) {
+    double mpi_time = 0;	/* time spent in MPI */
+
+    /* Print the current container stats */
+    printf("\n");
+    for (i = 0; i < depth; i++)
+      printf("   ");
+    printf("%s -- \t%d collective messages\n", p_cont->name, counter->nb_messages);
+
+#if 0
+    /* TODO: add statistics on the size of messages */
+    for (i = 0; i < depth; i++)
+      printf("   ");
+    printf("\tSize of messages (byte):");
+    __print_stat_int_counter(&counter->size, counter->nb_messages);
+    printf("\n");
+#endif
+
+    for (i = 0; i < depth; i++)
+      printf("   ");
+    printf("\tTime past in a collective operation (ms):");
+    __print_stat_double_counter(&counter->wait_duration, counter->nb_messages);
+    printf("\n");
+
+    /* call the container children so that they print their values */
+    for (i = 0; i < p_cont->nb_children; i++) {
+      __coll_stats_print_recurse(depth + 1, p_cont->children[i]);
+    }
+  }
+}
+
+/* print collective messages statistics */
+static void __coll_stats_print() {
+  int rank;
+  for (rank = 0; rank < NB_TRACES; rank++) {
+    p_eztrace_container p_cont = GET_PROCESS_CONTAINER(rank);
+    __coll_stats_print_recurse(0, p_cont);
+  }
+}
+
+/* compute collective messages statistics and print them */
+void print_coll_stats() {
+  __coll_stats_print();
+
+  __coll_print_communication_matrix_message_size();
+  __coll_print_communication_matrix_nb_messages();
+}
+
 /* compute point to point messages statistics and print them */
 void print_p2p_stats() {
-  __p2p_stats_reduce();
   __p2p_stats_print();
 
   dump_comm_matrix_nb_msg = 1;
@@ -417,11 +651,43 @@ void print_p2p_stats() {
   __p2p_print_communication_matrix_nb_messages();
 }
 
+
+/* compute point to point messages statistics */
+static void __stats_reduce() {
+  int rank;
+  eztrace_array_create(&__mpi_stats_freq, sizeof(struct __mpi_stats_freq_item), 32);
+
+  __mpi_stats_comm_matrix = malloc(sizeof(struct __mpi_stats_matrix_item*) * NB_TRACES);
+  __mpi_stats_comm_matrix_coll = malloc(sizeof(struct __mpi_stats_matrix_item*) * NB_TRACES);
+
+  for (rank = 0; rank < NB_TRACES; rank++) {
+    __mpi_stats_comm_matrix[rank] = malloc(sizeof(struct __mpi_stats_matrix_item) * NB_TRACES);
+    __mpi_stats_comm_matrix_coll[rank] = malloc(sizeof(struct __mpi_stats_matrix_item) * NB_TRACES);
+    int i;
+    for (i = 0; i < NB_TRACES; i++) {
+      __mpi_stats_comm_matrix[rank][i].total_len = 0;
+      __mpi_stats_comm_matrix[rank][i].nb_messages = 0;
+
+      __mpi_stats_comm_matrix_coll[rank][i].total_len = 0;
+      __mpi_stats_comm_matrix_coll[rank][i].nb_messages = 0;
+    }
+  }
+
+  for (rank = 0; rank < NB_TRACES; rank++) {
+    p_eztrace_container p_cont = GET_PROCESS_CONTAINER(rank);
+    __p2p_stats_reduce_recurse(0, rank, p_cont);
+    __coll_stats_reduce_recurse(0, rank, p_cont);
+  }
+}
+
 /* initialize MPI statistics module */
 void init_mpi_stats() {
   hierarchical_array_attach(EZTRACE_MPI_STATS_P2P_ID,
                             sizeof(struct mpi_p2p_stat_t));
 
+  hierarchical_array_attach(EZTRACE_MPI_STATS_COLL_ID,
+                            sizeof(struct mpi_coll_stat_t));
+
   /* todo: do the same for collective */
 }
 
@@ -481,3 +747,21 @@ void mpi_stats_plot_message_size() {
   free(data_path);
   free(gp_path);
 }
+
+
+void print_mpi_msg_stats() {
+  __stats_reduce();
+
+  print_p2p_stats();
+
+  printf("\n");
+  print_coll_stats();
+
+  printf("\n");
+  mpi_stats_plot_message_size();
+  printf("\n");
+
+  __all_print_communication_matrix_message_size();
+  __all_print_communication_matrix_nb_messages();
+
+}
