@@ -198,3 +198,8 @@ void mpif_dist_graph_create_adjacent_(MPI_Fint* comm_old, int* indegree,
 
   *comm_dist_graph = MPI_Comm_c2f(comm_dist_graph_c);
 }
+
+void mpif_type_size_(int* datatype, int*size, MPI_Fint *error) {
+  *error = MPI_Type_size(*datatype, size);
+}
+
