@@ -36,7 +36,11 @@ static void MPI_Scatterv_prolog(CONST void *sendbuf __attribute__((unused)),
   libMPI_Comm_size(comm, &size);
   libMPI_Comm_rank(comm, &rank);
 
-  EZTRACE_EVENT_PACKED_3(EZTRACE_MPI_START_Scatterv, (app_ptr)comm, size, rank);
+  int ssize;
+  MPI_Type_size(sendtype, &ssize);
+  int data_size = ssize * sendcnts[0];
+
+  EZTRACE_EVENT_PACKED_4(EZTRACE_MPI_START_Scatterv, (app_ptr)comm, size, rank, data_size);
 }
 
 static int MPI_Scatterv_core(CONST void *sendbuf,
@@ -65,7 +69,6 @@ static void MPI_Scatterv_epilog(CONST void *sendbuf __attribute__((unused)),
 {
   int rank = -1;
   int size = -1;
-  int i = 0;
   libMPI_Comm_size(comm, &size);
   libMPI_Comm_rank(comm, &rank);
 
@@ -103,12 +106,12 @@ void mpif_scatterv_(void *sbuf, int *scount, int *displs, MPI_Fint *sd,
   void *c_sbuf = CHECK_MPI_IN_PLACE(sbuf);
   void *c_rbuf = CHECK_MPI_IN_PLACE(rbuf);
 
-  MPI_Scatterv_prolog(sbuf, scount, displs, c_stype, c_rbuf, *rcount, c_rtype,
+  MPI_Scatterv_prolog(c_sbuf, scount, displs, c_stype, c_rbuf, *rcount, c_rtype,
                       *root, c_comm);
 
-  *error = MPI_Scatterv_core(sbuf, scount, displs, c_stype, c_rbuf, *rcount,
+  *error = MPI_Scatterv_core(c_sbuf, scount, displs, c_stype, c_rbuf, *rcount,
                              c_rtype, *root, c_comm);
 
-  MPI_Scatterv_epilog(sbuf, scount, displs, c_stype, c_rbuf, *rcount, c_rtype,
+  MPI_Scatterv_epilog(c_sbuf, scount, displs, c_stype, c_rbuf, *rcount, c_rtype,
                       *root, c_comm);
 }
