@@ -1,6 +1,6 @@
 /* -*- c-file-style: "GNU" -*- */
 /*
- * Copyright © CNRS, INRIA, Université Bordeaux 1
+ * Copyright (C) CNRS, INRIA, Universite Bordeaux 1, Telecom SudParis
  * See COPYING in top-level directory.
  */
 
@@ -21,7 +21,6 @@
 #include "mpi_ev_codes.h"
 #include "eztrace.h"
 
-
 static void MPI_Send_prolog(CONST void *buf __attribute__((unused)),
 			    int count,
 			    MPI_Datatype datatype,
@@ -32,11 +31,11 @@ static void MPI_Send_prolog(CONST void *buf __attribute__((unused)),
   int size;
   /* retrieve the size of the datatype so that we can compute the message length */
   MPI_Type_size(datatype, &size);
-  EZTRACE_EVENT4 (FUT_MPI_START_SEND, count*size, dest, tag, comm);
+   EZTRACE_EVENT_PACKED_4(EZTRACE_MPI_START_SEND, count*size, dest, tag, (app_ptr)comm);
 }
 
-static int MPI_Send_core(CONST void *buf, int count, MPI_Datatype datatype,int dest, int tag, MPI_Comm comm)
-{
+static int MPI_Send_core(CONST void *buf, int count, MPI_Datatype datatype,
+                         int dest, int tag, MPI_Comm comm) {
   return libMPI_Send(buf, count, datatype, dest, tag, comm);
 }
 
@@ -47,12 +46,12 @@ static void MPI_Send_epilog(CONST void *buf __attribute__((unused)),
 			    int tag,
 			    MPI_Comm comm)
 {
-  EZTRACE_EVENT3 (FUT_MPI_STOP_SEND, dest, tag, comm);
+    EZTRACE_EVENT_PACKED_3(EZTRACE_MPI_STOP_SEND, dest, tag, (app_ptr)comm);
 }
 
 /* C function */
-int MPI_Send(CONST void *buf, int count, MPI_Datatype datatype,int dest, int tag, MPI_Comm comm)
-{
+int MPI_Send(CONST void *buf, int count, MPI_Datatype datatype, int dest,
+             int tag, MPI_Comm comm) {
   FUNCTION_ENTRY;
   MPI_Send_prolog(buf, count, datatype, dest, tag, comm);
   int ret = MPI_Send_core(buf, count, datatype, dest, tag, comm);
@@ -61,9 +60,9 @@ int MPI_Send(CONST void *buf, int count, MPI_Datatype datatype,int dest, int tag
 }
 
 /* fortran function */
-void mpif_send_(void* buf, int *count, MPI_Fint *d,
-		int *dest, int *tag, MPI_Fint *c, int *error)
-{
+void mpif_send_(void* buf, int *count, MPI_Fint *d, int *dest, int *tag,
+                MPI_Fint *c, int *error) {
+  FUNCTION_ENTRY;
   MPI_Comm c_comm = MPI_Comm_f2c(*c);
   MPI_Datatype c_type = MPI_Type_f2c(*d);
   MPI_Send_prolog(buf, *count, c_type, *dest, *tag, c_comm);
