@@ -16,18 +16,6 @@
 #include "eztrace_stack.h"
 #include "eztrace_convert_pthread.h"
 
-struct pthread_thread_info_t {
-  double time_sem_wait;		/* time spent in sem_wait */
-  double time_spin_lock;	/* time spent in sepin_lock */
-  double time_mutex_lock;	/* time spent in mutex_lock */
-  double time_cond_wait;	/* time spent in cond_wait */
-  double time_rwlock_rdlock;	/* time spent in rwlock_rdlock */
-  double time_rwlock_wrlock;	/* time spent in rwlock_wrlock */
-  double time_barrier;		/* time spent in barrier */
-
-  double enter_last_lock_time;	/* timestamp when the thread entered the last lock */
-  struct thread_info_t *p_thread;
-};
 
 /* add a hook in the thread structure in order to store information
  * about pending parallel sections
@@ -37,15 +25,13 @@ struct pthread_thread_info_t *__register_thread_hook(int tid)
   DECLARE_CUR_PROCESS(p_process);
   struct pthread_thread_info_t *p_info = (struct pthread_thread_info_t*) malloc(sizeof(struct pthread_thread_info_t));
   p_info->p_thread = GET_THREAD_INFO(CUR_INDEX, tid);
-  p_info->enter_last_lock_time = 0;
 
-  p_info->time_sem_wait = 0;
-  p_info->time_spin_lock = 0;
-  p_info->time_mutex_lock = 0;
-  p_info->time_cond_wait = 0;
-  p_info->time_rwlock_rdlock = 0;
-  p_info->time_rwlock_wrlock = 0;
-  p_info->time_barrier = 0;
+  int i;
+  for(i=0; i<NB_LOCK_TYPES; i++) {
+    p_info->duration_acquire[i] = 0;
+    p_info->start_acquire[i] = 0;
+    p_info->duration_critical_section[i] = 0;
+  }
 
   /* add the hook in the thread info structure */
   ezt_hook_list_add(&p_info->p_thread->hooks, p_info, (uint8_t)EZTRACE_PTHREAD_EVENTS_ID);
@@ -82,11 +68,15 @@ void handle_start_sem_wait(void) {
 
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
+
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, SEMAPHORE);
+  record_lock_start_acquire(lock_info, p_info);
 
   PTHREAD_CHANGE()
     pushState(CURRENT, "ST_Thread", thread_id, "STV_Blocked_sem");
@@ -97,6 +87,8 @@ void handle_stop_sem_wait(void) {
 
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   int retval;
@@ -104,9 +96,7 @@ void handle_stop_sem_wait(void) {
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, SEMAPHORE);
-
-  lock_info->nb_acquire++;
-  lock_info->last_owner_tid = CUR_THREAD_ID;
+  record_lock_acquired(lock_info, p_info);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -120,12 +110,15 @@ void handle_spin_lock_start() {
 
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, SPINLOCK);
+  record_lock_start_acquire(lock_info, p_info);
 
   PTHREAD_CHANGE()
     pushState(CURRENT, "ST_Thread", thread_id, "STV_Blocked_spin");
@@ -135,6 +128,8 @@ void handle_spin_lock_stop() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   int retval;
@@ -142,7 +137,7 @@ void handle_spin_lock_stop() {
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, SPINLOCK);
-  record_lock_acquired(lock_info);
+  record_lock_acquired(lock_info, p_info);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -152,6 +147,8 @@ void handle_spin_lock_stop() {
 
 void handle_spin_trylock() {
   FUNC_NAME;
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   int result;
@@ -163,7 +160,8 @@ void handle_spin_trylock() {
 
     struct pthread_lock_info_t *lock_info = NULL;
     GET_LOCK_INFO(lock_info, p_process, lock_ptr, SPINLOCK);
-    record_lock_acquired(lock_info);
+    record_lock_start_acquire(lock_info, p_info);
+    record_lock_acquired(lock_info, p_info);
 
     PTHREAD_CHANGE()
       pushState(CURRENT, "ST_Thread", thread_id, "STV_Critical_spin");
@@ -174,6 +172,8 @@ void handle_spin_unlock() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
@@ -181,8 +181,8 @@ void handle_spin_unlock() {
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, SPINLOCK);
 
-  record_lock_release(lock_info);
-  __check_lock_mismatch(lock_info, p_process, CUR_THREAD_ID);
+  record_lock_release(lock_info, p_info);
+  //  __check_lock_mismatch(lock_info, p_process, CUR_THREAD_ID);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -199,10 +199,12 @@ void handle_mutex_trylock(int result) {
 
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, MUTEX);
-  record_lock_acquired(lock_info);
+  record_lock_acquired(lock_info, p_info);
 
   PTHREAD_CHANGE()
     pushState(CURRENT, "ST_Thread", thread_id, "STV_Critical_mutex");
@@ -212,12 +214,15 @@ void handle_mutex_lock_start() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, MUTEX);
+  record_lock_start_acquire(lock_info, p_info);
 
   PTHREAD_CHANGE()
     pushState(CURRENT, "ST_Thread", thread_id, "STV_Blocked_mutex");
@@ -228,6 +233,8 @@ void handle_mutex_lock_stop() {
 
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   int retval;
@@ -235,7 +242,7 @@ void handle_mutex_lock_stop() {
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, MUTEX);
-  record_lock_acquired(lock_info);
+  record_lock_acquired(lock_info, p_info);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -248,14 +255,16 @@ void handle_mutex_unlock() {
 
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, MUTEX);
-  record_lock_release(lock_info);
-  __check_lock_mismatch(lock_info, p_process, CUR_THREAD_ID);
+  record_lock_release(lock_info, p_info);
+  //  __check_lock_mismatch(lock_info, p_process, CUR_THREAD_ID);
 
   PTHREAD_CHANGE()
     addEvent(CURRENT, "E_Mutex_Unlock", thread_id, lock_info->info);
@@ -275,10 +284,16 @@ void handle_cond_broadcast() {
 void handle_cond_start_wait() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
+  DECLARE_CUR_PROCESS(p_process);
   DECLARE_CUR_THREAD(p_thread);
   INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
-  assert(p_info->enter_last_lock_time == 0);
-  p_info->enter_last_lock_time = CURRENT;
+
+  app_ptr lock_ptr;
+  GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
+
+  struct pthread_lock_info_t *lock_info = NULL;
+  GET_LOCK_INFO(lock_info, p_process, lock_ptr, CONDITION);
+  record_lock_start_acquire(lock_info, p_info);
 
   PTHREAD_CHANGE() pushState(CURRENT, "ST_Thread", thread_id, "STV_Blocked_cond");
 }
@@ -287,13 +302,15 @@ void handle_cond_stop_wait() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, CONDITION);
-  record_lock_acquired(lock_info);
+  record_lock_acquired(lock_info, p_info);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -306,14 +323,13 @@ void handle_rwlock_rdlock_start() {
   DECLARE_CUR_PROCESS(p_process);
   DECLARE_CUR_THREAD(p_thread);
   INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
-  assert(p_info->enter_last_lock_time == 0);
-  p_info->enter_last_lock_time = CURRENT;
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, RWLOCK);
+  record_lock_start_acquire(lock_info, p_info);
 
   PTHREAD_CHANGE()
     pushState(CURRENT, "ST_Thread", thread_id, "STV_Blocked_rwlock");
@@ -323,19 +339,15 @@ void handle_rwlock_rdlock_stop() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
-
   DECLARE_CUR_THREAD(p_thread);
   INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
-  assert(p_info->enter_last_lock_time > 0);
-  p_info->time_rwlock_rdlock += (CURRENT-p_info->enter_last_lock_time);
-  p_info->enter_last_lock_time = 0;
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, RWLOCK);
-  record_lock_acquired(lock_info);
+  record_lock_acquired(lock_info, p_info);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -347,12 +359,15 @@ void handle_rwlock_wrlock_start() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, RWLOCK);
+  record_lock_start_acquire(lock_info, p_info);
 
   PTHREAD_CHANGE()
     pushState(CURRENT, "ST_Thread", thread_id, "STV_Blocked_rwlock");
@@ -362,13 +377,15 @@ void handle_rwlock_wrlock_stop() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
 
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, RWLOCK);
-  record_lock_acquired(lock_info);
+  record_lock_acquired(lock_info, p_info);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -380,6 +397,8 @@ void handle_rwlock_unlock() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
   DECLARE_CUR_PROCESS(p_process);
+  DECLARE_CUR_THREAD(p_thread);
+  INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
 
   app_ptr lock_ptr;
   GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
@@ -387,8 +406,8 @@ void handle_rwlock_unlock() {
   struct pthread_lock_info_t *lock_info = NULL;
   GET_LOCK_INFO(lock_info, p_process, lock_ptr, RWLOCK);
 
-  record_lock_release(lock_info);
-  __check_lock_mismatch(lock_info, p_process, CUR_THREAD_ID);
+  record_lock_release(lock_info, p_info);
+  //  __check_lock_mismatch(lock_info, p_process, CUR_THREAD_ID);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -397,11 +416,15 @@ void handle_rwlock_unlock() {
 void handle_barrier_start() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
-
+  DECLARE_CUR_PROCESS(p_process);
   DECLARE_CUR_THREAD(p_thread);
   INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
-  assert(p_info->enter_last_lock_time == 0);
-  p_info->enter_last_lock_time = CURRENT;
+
+  app_ptr lock_ptr;
+  GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
+  struct pthread_lock_info_t *lock_info = NULL;
+  GET_LOCK_INFO(lock_info, p_process, lock_ptr, BARRIER);
+  record_lock_start_acquire(lock_info, p_info);
 
   PTHREAD_CHANGE() pushState(CURRENT, "ST_Thread", thread_id, "STV_Blocked_barrier");
 }
@@ -409,12 +432,16 @@ void handle_barrier_start() {
 void handle_barrier_stop() {
   FUNC_NAME;
   DECLARE_THREAD_ID_STR(thread_id, CUR_INDEX, CUR_THREAD_ID);
-
+  DECLARE_CUR_PROCESS(p_process);
   DECLARE_CUR_THREAD(p_thread);
   INIT_PTHREAD_THREAD_INFO(p_thread, p_info);
-  assert(p_info->enter_last_lock_time > 0);
-  p_info->time_barrier += (CURRENT-p_info->enter_last_lock_time);
-  p_info->enter_last_lock_time = 0;
+
+  app_ptr lock_ptr;
+  GET_PARAM_PACKED_1(CUR_EV, lock_ptr);
+  struct pthread_lock_info_t *lock_info = NULL;
+  GET_LOCK_INFO(lock_info, p_process, lock_ptr, BARRIER);
+  record_lock_acquired(lock_info, p_info);
+  record_lock_release(lock_info, p_info);
 
   PTHREAD_CHANGE()
     popState(CURRENT, "ST_Thread", thread_id);
@@ -575,7 +602,7 @@ void print_pthread_stats() {
           &p_process->hooks, (uint8_t) EZTRACE_PTHREAD_EVENTS_ID);
 
     if (!p_info)
-      continue; /* No gomp process info attached, skip this process */
+      continue;
 
     uint32_t nb_acquire = 0;
     uint32_t nb_locks = 0;
@@ -588,30 +615,17 @@ void print_pthread_stats() {
         (struct pthread_lock_info_t *) token->data;
       nb_acquire += lock->nb_acquire;
       nb_locks++;
-      switch (lock->lock) {
-      case MUTEX:
-        printf("\tmutex");
-        break;
-      case SPINLOCK:
-        printf("\tspinlock");
-        break;
-      case RWLOCK:
-        printf("\trwlock");
-        break;
-      case SEMAPHORE:
-        printf("\tsemaphore");
-        break;
-      case BARRIER:
-        printf("\tbarrier");
-        break;
-      case CONDITION:
-        printf("\tcondition");
-        break;
-      };
-      printf(" 0x%p was acquired %d times.", (void*)lock->ptr, lock->nb_acquire);
+      char lock_type[80];
+      __lock_type_to_str(lock_type, lock->lock);
+      printf("\t%s 0x%p was acquired %d times.", lock_type, (void*)lock->ptr, lock->nb_acquire);
+
       if (lock->duration_critical_section)
         printf(" total duration of critical_sections: %lf ms.",
                lock->duration_critical_section);
+
+      if (lock->duration_acquire)
+        printf(" total time spent waiting: %lf ms.",
+               lock->duration_acquire);
       printf("\n");
     }
     printf("Total: %d locks acquired %d times\n", nb_locks, nb_acquire);
@@ -619,14 +633,14 @@ void print_pthread_stats() {
     __free_lock_info_list(p_info);
   }
 
-  double total_time_sem_wait      = 0;		/* time spent in sem_wait */
-  double total_time_spin_lock     = 0;	/* time spent in sepin_lock */
-  double total_time_mutex_lock    = 0;	/* time spent in mutex_lock */
-  double total_time_cond_wait     = 0;	/* time spent in cond_wait */
-  double total_time_rwlock_rdlock = 0;	/* time spent in rwlock_rdlock */
-  double total_time_rwlock_wrlock = 0;	/* time spent in rwlock_wrlock */
-  double total_time_barrier       = 0;		/* time spent in barrier */
 
+  double total_duration_acquire[NB_LOCK_TYPES]; /* time spent in waiting for a lock */
+  double total_duration_critical_section[NB_LOCK_TYPES]; /* time spent in a critical section */
+  int l;
+  for(l=0; l<NB_LOCK_TYPES; l++) {
+    total_duration_acquire[l] = 0;
+    total_duration_critical_section[l] = 0;
+  }
   for(i=0; i<NB_TRACES; i++) {
     unsigned j;
     struct eztrace_container_t* p_cont = GET_PROCESS_CONTAINER(i);
@@ -638,83 +652,53 @@ void print_pthread_stats() {
       if(!t_info)
 	continue;
 
-      if((!t_info->time_sem_wait     ) &&
-	 (!t_info->time_spin_lock    ) &&
-	 (!t_info->time_mutex_lock   ) &&
-	 (!t_info->time_cond_wait    ) &&
-	 (!t_info->time_rwlock_rdlock) &&
-	 (!t_info->time_rwlock_wrlock) &&
-	 (!t_info->time_barrier))
+      int l;
+      int should_skip=1;
+      for(l=0; l<NB_LOCK_TYPES; l++) {
+	if(t_info->duration_acquire[l] || t_info->duration_critical_section[l]) {
+	  should_skip=0;
+	}
+      }
+
+      if(should_skip) {
 	continue;
+      }
 
       printf("\nThread %s\n", p_thread->name);
+      for(l=0; l<NB_LOCK_TYPES; l++) {
+	if(t_info->duration_acquire[l]) {
+	  char lock_type[80];
+	  __lock_type_to_str(lock_type, l);
+	  printf("\ttime spent waiting on a %s: %lf ms\n", lock_type, t_info->duration_acquire[l]);
+	  total_duration_acquire[l] += t_info->duration_acquire[l];
+	}
 
-      if(t_info->time_sem_wait) {
-	printf("\ttime spent in sem_wait  : %lf ms\n", t_info->time_sem_wait);
-	total_time_sem_wait += t_info->time_sem_wait;
-      }
-
-      if(t_info->time_spin_lock) {
-	printf("\ttime spent in spin_lock : %lf ms\n", t_info->time_spin_lock);
-	total_time_spin_lock += t_info->time_spin_lock;
-      }
-
-      if(t_info->time_mutex_lock) {
-	printf("\ttime spent in mutex_lock: %lf ms\n", t_info->time_mutex_lock);
-	total_time_mutex_lock += t_info->time_mutex_lock;
-      }
-
-      if(t_info->time_cond_wait) {
-	printf("\ttime spent in cond_wait : %lf ms\n", t_info->time_cond_wait);
-	total_time_cond_wait += t_info->time_cond_wait;
-      }
-
-      if(t_info->time_rwlock_rdlock) {
-	printf("\ttime spent in rwlock_rd : %lf ms\n", t_info->time_rwlock_rdlock);
-	total_time_rwlock_rdlock += t_info->time_rwlock_rdlock;
-      }
-
-      if(t_info->time_rwlock_wrlock) {
-	printf("\ttime spent in rwlock_wr : %lf ms\n", t_info->time_rwlock_wrlock);
-	total_time_rwlock_wrlock += t_info->time_rwlock_wrlock;
-      }
-
-      if(t_info->time_barrier) {
-	printf("\ttime spent in barrier   : %lf ms\n", t_info->time_barrier);
-	total_time_barrier += t_info->time_barrier;
+	if(t_info->duration_critical_section[l]) {
+	  char lock_type[80];
+	  __lock_type_to_str(lock_type, l);
+	  printf("\ttime spent in a %s critical: %lf ms\n", lock_type,
+		 t_info->duration_critical_section[l]);
+	  total_duration_critical_section[l] += t_info->duration_critical_section[l];
+	}
       }
     }
 
     printf("\nTotal for %s\n", p_cont->name);
-      if(total_time_sem_wait) {
-	printf("\ttime spent in sem_wait  : %lf ms\n", total_time_sem_wait);
+    int l;
+    for(l=0; l<NB_LOCK_TYPES; l++) {
+      if(total_duration_acquire[l]) {
+	char lock_type[80];
+	__lock_type_to_str(lock_type, l);
+	printf("\ttime spent waiting on a %s: %lf ms\n", lock_type, total_duration_acquire[l]);
       }
-
-      if(total_time_spin_lock) {
-	printf("\ttime spent in spin_lock : %lf ms\n", total_time_spin_lock);
-      }
-
-      if(total_time_mutex_lock) {
-	printf("\ttime spent in mutex_lock: %lf ms\n", total_time_mutex_lock);
-      }
-
-      if(total_time_cond_wait) {
-	printf("\ttime spent in cond_wait : %lf ms\n", total_time_cond_wait);
-      }
-
-      if(total_time_rwlock_rdlock) {
-	printf("\ttime spent in rwlock_rd : %lf ms\n", total_time_rwlock_rdlock);
-      }
-
-      if(total_time_rwlock_wrlock) {
-	printf("\ttime spent in rwlock_wr : %lf ms\n", total_time_rwlock_wrlock);
-      }
-
-      if(total_time_barrier) {
-	printf("\ttime spent in barrier   : %lf ms\n", total_time_barrier);
+      if(total_duration_critical_section[l]) {
+	char lock_type[80];
+	__lock_type_to_str(lock_type, l);
+	printf("\ttime spent in a %s critical: %lf ms\n", lock_type,
+	       total_duration_critical_section[l]);
       }
+    }
   }
-
 }
 
 struct eztrace_convert_module pthread_module;
