@@ -6,7 +6,7 @@
 
 #define eztrace_extra "OMP_NUM_THREADS=2 "
 #define nproc 1
-#define nameprog "test/openmp/simple"
+#define nameprog "test/openmp/openmp_simple"
 #define NBMAXEVENT 55
 #define NBMAXREALEVENT 85
 #define module "omp"
